#pragma once

// C++ system level
#include <set>
#include <cstring>     // memset, strncmp
// #include <cstdlib>     // TODO: is this for memcpy?
#include <cinttypes>
// #include <functional>

// ESP32 specific
// #include "esp_err.h"
#include "nvs_flash.h"
#include "esp_log.h"
// #include "driver/gpio.h"
// #include "driver/uart.h"

// project specific
#include "SpiderLib/NumberTypes.hpp"
// #include "SpiderLib/ManagedMutex.hpp"

// qthing stuff
// #include <qthing>


namespace SiliconTorch {

  namespace NVSExplorer {


    extern const char* TAG;


    class NVSExplorer {

      public:

        std::set<std::string> listNamespaces();
        std::set<std::string> listKeys(const std::string& nameSpace);

        void rmKey(const std::string& nameSpace, const std::string& key);


        // Gets the key regardless of its type in flash
        i64  getSignedInt(const std::string& nameSpace, const std::string& key, i64 defaultValue = 0);
        u64 getUnsignedInt(const std::string& nameSpace, const std::string& key, u64 defaultValue = 0);

        float  getFloat(const std::string& nameSpace, const std::string& key);
        double getDouble(const std::string& nameSpace, const std::string& key);

        bool setFloat(const std::string& nameSpace, const std::string& key, float value);
        bool setDouble(const std::string& nameSpace, const std::string& key, double value);




        std::string deviceTopic(const std::string& suffix);

        std::string requestTopic(const std::string& cmd);
        std::string requestTopic(const std::string& cmd, const std::string& nameSpace);
        std::string requestTopic(const std::string& cmd, const std::string& nameSpace, const std::string& key);

        std::string responseTopic(const std::string& cmd);
        std::string responseTopic(const std::string& cmd, const std::string& nameSpace);
        std::string responseTopic(const std::string& cmd, const std::string& nameSpace, const std::string& key);

        // ⬇⬇ Singleton stuff ⬇⬇

        static NVSExplorer& instance() {
          static NVSExplorer _instance;
          return _instance;
        }

        NVSExplorer(NVSExplorer const&)    = delete;
        void operator=(NVSExplorer const&) = delete;

        // ⬆⬆ Singleton stuff ⬆⬆

      private:

        NVSExplorer();

    };


  }
}
