NVSExplorer
===========

A simple tool for exploring the `NVS` on `ESP32` platform through a high-level `C++` API.

All API endpoints are also mapped to `MQTT` topics to provide a some kind of a remote administration toolkit for `ESP32` devices.


`MQTT` API
----------

`NVSExplorer` has its topics living inside the local `qthing` topic scheme.

The base-topic thus is `device/<NAME>/NVS` where `<NAME>` is the device name.
This topic will further be referred to just as `BASE`, like in `BASE/cmd`.

The request/response scheme doesn't follow the usual property ones as this would lead to publish loops.
Therefore we establish distinct request/response topics.

- `request` topic:
  `BASE/rq`

- `response` topic:
  `BASE/rs`  (the `response`-topic *may* change in the future as I'm certainly unclear about it)

This topics will further on be referred to as `REQUEST` and `RESPONSE`, as in `REQUEST/ls`.


### Commands

The following commands are or will be supported by `NVSExplorer`.
As for the topic convention just swap `REQUEST` and `RESPONSE` to get your result.
Not all commands will publish a result, but there may be an `ERROR`-topic in the future to indicate failure or success of such commands.

In the following list, `argument` stands for the message content received on the topic, `NS` and `KEY` denote the used namespace and key.

- `ls` - **list namespaces**  
  request: `REQUEST/ls`  
  response: `RESPONSE/ls`  
  argument: used as delimiter for concatenating the list (default: `' '`)  
  description: returns a list of the namespaces found in `NVS`

- `ls` - **list keys**  
  request: `REQUEST/ls/NS`  
  response: `RESPONSE/ls/NS`  
  argument: used as delimiter for concatenating the list (default: `' '`)  
  description: returns a list of the keys found in namespace `NS`

- `get` - **get generic value**  
  request: `REQUEST/get/NS/KEY`  
  response: `RESPONSE/get/NS/KEY`  
  description: reads the value and returns it in its natural form

- `get_raw` - **get generic value**  
  request: `REQUEST/get_raw/NS/KEY`  
  response: `RESPONSE/get_raw/NS/KEY`  
  description: returns the value in its binary form, for example `2 bytes` in case of `u16`

- `get_<float|double>` - **get floating point value**  
  request: `REQUEST/get_<float|double>/NS/KEY`  
  response: `RESPONSE/get_<float|double>/NS/KEY`  
  description: reads the value, `reinterpret_cast`s it to `float` or `double` and returns it converted to string



### Types

#### TODO
